@@ -21,7 +21,6 @@ from openstack.cloud import _utils
 from openstack.cloud import exc
 from openstack.cloud import meta
 from openstack.compute.v2._proxy import Proxy
-from openstack.compute.v2 import quota_set as _qs
 from openstack.compute.v2 import server as _server
 from openstack import exceptions
 from openstack import utils
@@ -99,8 +98,9 @@ class ComputeCloudMixin:
         :param string include: If given, will return a flavor whose name
             contains this string as a substring.
         :param get_extra:
+
         :returns: A compute ``Flavor`` object.
-        :raises: :class:`~openstack.exceptions.OpenStackCloudException` if no
+        :raises: :class:`~openstack.exceptions.SDKException` if no
             matching flavour could be found.
         """
         flavors = self.list_flavors(get_extra=get_extra)
@@ -109,14 +109,14 @@ class ComputeCloudMixin:
                 not include or include in flavor['name']
             ):
                 return flavor
-        raise exc.OpenStackCloudException(
+        raise exceptions.SDKException(
             "Could not find a flavor with {ram} and '{include}'".format(
                 ram=ram, include=include
             )
         )
 
     def _nova_extensions(self):
-        extensions = set([e.alias for e in self.compute.extensions()])
+        extensions = {e.alias for e in self.compute.extensions()}
         return extensions
 
     def _has_nova_extension(self, extension_name):
@@ -175,10 +175,11 @@ class ComputeCloudMixin:
 
         :param name_or_id: Name or unique ID of the server group(s).
         :param filters: A dict containing additional filters to use.
+
         :returns: A list of compute ``ServerGroup`` objects matching the search
             criteria.
-        :raises: ``OpenStackCloudException``: if something goes wrong during
-            the OpenStack API call.
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call.
         """
         server_groups = self.list_server_groups()
         return _utils._filter_list(server_groups, name_or_id, filters)
@@ -283,7 +284,8 @@ class ComputeCloudMixin:
 
         :returns: False if server or security groups are undefined, True
             otherwise.
-        :raises: ``OpenStackCloudException``, on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         server, security_groups = self._get_server_security_groups(
             server, security_groups
@@ -306,7 +308,8 @@ class ComputeCloudMixin:
 
         :returns: False if server or security groups are undefined, True
             otherwise.
-        :raises: ``OpenStackCloudException``, on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         server, security_groups = self._get_server_security_groups(
             server, security_groups
@@ -378,18 +381,21 @@ class ComputeCloudMixin:
 
         :param name_or_id: (optional) project name or ID to get limits for
             if different from the current project
-        :raises: OpenStackCloudException if it's not a valid project
+
         :returns: A compute
             :class:`~openstack.compute.v2.limits.Limits.AbsoluteLimits` object.
+        :raises: :class:`~openstack.exceptions.SDKException` if it's not a
+            valid project
         """
         params = {}
-        project_id = None
         if name_or_id:
-            proj = self.get_project(name_or_id)
-            if not proj:
-                raise exc.OpenStackCloudException("project does not exist")
-            project_id = proj.id
-            params['tenant_id'] = project_id
+            project = self.identity.find_project(name_or_id)
+            if not project:
+                raise exceptions.SDKException(
+                    f"Project {name_or_id} was requested but was not found "
+                    f"on the cloud"
+                )
+            params['tenant_id'] = project.id
         return self.compute.get_limits(**params).absolute
 
     def get_keypair(self, name_or_id, filters=None):
@@ -468,21 +474,21 @@ class ComputeCloudMixin:
 
         :returns: A string containing the text of the console log or an
             empty string if the cloud does not support console logs.
-        :raises: OpenStackCloudException if an invalid server argument is given
-            or if something else unforseen happens
+        :raises: :class:`~openstack.exceptions.SDKException` if an invalid
+            server argument is given or if something else unforseen happens
         """
 
         if not isinstance(server, dict):
             server = self.get_server(server, bare=True)
 
         if not server:
-            raise exc.OpenStackCloudException(
+            raise exceptions.SDKException(
                 "Console log requested for invalid server"
             )
 
         try:
             return self._get_server_console_output(server['id'], length)
-        except exc.OpenStackCloudBadRequest:
+        except exceptions.BadRequestException:
             return ""
 
     def _get_server_console_output(self, server_id, length=None):
@@ -582,8 +588,10 @@ class ComputeCloudMixin:
 
         :param name: Name of the keypair being created.
         :param public_key: Public key for the new keypair.
+
         :returns: The created compute ``Keypair`` object.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         keypair = {
             'name': name,
@@ -598,8 +606,8 @@ class ComputeCloudMixin:
         :param name: Name of the keypair to delete.
 
         :returns: True if delete succeeded, False otherwise.
-
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         try:
             self.compute.delete_keypair(name, ignore_missing=False)
@@ -630,13 +638,15 @@ class ComputeCloudMixin:
         :param wait: If true, waits for image to be created.
         :param timeout: Seconds to wait for image creation. None is forever.
         :param metadata: Metadata to give newly-created image entity
+
         :returns: The created image ``Image`` object.
-        :raises: OpenStackCloudException if there are problems uploading
+        :raises: :class:`~openstack.exceptions.SDKException` if there are
+            problems uploading
         """
         if not isinstance(server, dict):
             server_obj = self.get_server(server, bare=True)
             if not server_obj:
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     "Server {server} could not be found and therefore"
                     " could not be snapshotted.".format(server=server)
                 )
@@ -703,6 +713,7 @@ class ComputeCloudMixin:
         'config_drive',
         'admin_pass',
         'disk_config',
+        'tags',
     )
     def create_server(
         self,
@@ -800,8 +811,10 @@ class ComputeCloudMixin:
         :param group: ServerGroup dict, name or id to boot the server in.
             If a group is provided in both scheduler_hints and in the group
             param, the group param will win. (Optional, defaults to None)
+
         :returns: The created compute ``Server`` object.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         # TODO(shade) Image is optional but flavor is not - yet flavor comes
         # after image in the argument list. Doh.
@@ -838,9 +851,9 @@ class ComputeCloudMixin:
                 kwargs[desired] = value
 
         if group:
-            group_obj = self.get_server_group(group)
+            group_obj = self.compute.find_server_group(group)
             if not group_obj:
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     "Server Group {group} was requested but was not found"
                     " on the cloud".format(group=group)
                 )
@@ -856,7 +869,7 @@ class ComputeCloudMixin:
                 # Be nice and help the user out
                 kwargs['nics'] = [kwargs['nics']]
             else:
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     'nics parameter to create_server takes a list of dicts.'
                     ' Got: {nics}'.format(nics=kwargs['nics'])
                 )
@@ -869,9 +882,9 @@ class ComputeCloudMixin:
                 if isinstance(net_name, dict) and 'id' in net_name:
                     network_obj = net_name
                 else:
-                    network_obj = self.get_network(name_or_id=net_name)
+                    network_obj = self.network.find_network(net_name)
                 if not network_obj:
-                    raise exc.OpenStackCloudException(
+                    raise exceptions.SDKException(
                         'Network {network} is not a valid network in'
                         ' {cloud}:{region}'.format(
                             network=network,
@@ -897,9 +910,9 @@ class ComputeCloudMixin:
                 nic.pop('net-name', None)
             elif 'net-name' in nic:
                 net_name = nic.pop('net-name')
-                nic_net = self.get_network(net_name)
+                nic_net = self.network.find_network(net_name)
                 if not nic_net:
-                    raise exc.OpenStackCloudException(
+                    raise exceptions.SDKException(
                         "Requested network {net} could not be found.".format(
                             net=net_name
                         )
@@ -908,7 +921,7 @@ class ComputeCloudMixin:
             for ip_key in ('v4-fixed-ip', 'v6-fixed-ip', 'fixed_ip'):
                 fixed_ip = nic.pop(ip_key, None)
                 if fixed_ip and net.get('fixed_ip'):
-                    raise exc.OpenStackCloudException(
+                    raise exceptions.SDKException(
                         "Only one of v4-fixed-ip, v6-fixed-ip or fixed_ip"
                         " may be given"
                     )
@@ -923,7 +936,7 @@ class ComputeCloudMixin:
                 utils.require_microversion(self.compute, '2.42')
                 net['tag'] = nic.pop('tag')
             if nic:
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     "Additional unsupported keys given for server network"
                     " creation: {keys}".format(keys=nic.keys())
                 )
@@ -937,10 +950,21 @@ class ComputeCloudMixin:
                 kwargs['networks'] = 'auto'
 
         if image:
+            # TODO(stephenfin): Drop support for dicts: we should only accept
+            # strings or Image objects
             if isinstance(image, dict):
                 kwargs['imageRef'] = image['id']
             else:
-                kwargs['imageRef'] = self.get_image(image).id
+                image_obj = self.image.find_image(image)
+                if not image_obj:
+                    raise exc.OpenStackCloudException(
+                        f"Image {image} was requested but was not found "
+                        f"on the cloud"
+                    )
+                kwargs['imageRef'] = image_obj.id
+
+        # TODO(stephenfin): Drop support for dicts: we should only accept
+        # strings or Image objects
         if isinstance(flavor, dict):
             kwargs['flavorRef'] = flavor['id']
         else:
@@ -971,10 +995,26 @@ class ComputeCloudMixin:
         admin_pass = server.admin_password or kwargs.get('admin_pass')
         if not wait:
             server = self.compute.get_server(server.id)
-            if server.status == 'ERROR':
-                raise exc.OpenStackCloudCreateException(
-                    resource='server', resource_id=server.id
+            if server['status'] == 'ERROR':
+                if (
+                    'fault' in server
+                    and server['fault'] is not None
+                    and 'message' in server['fault']
+                ):
+                    raise exceptions.SDKException(
+                        "Error in creating the server. "
+                        "Compute service reports fault: {reason}".format(
+                            reason=server['fault']['message']
+                        ),
+                        extra_data=dict(server=server),
+                    )
+
+                raise exceptions.SDKException(
+                    "Error in creating the server "
+                    "(no further information available)",
+                    extra_data=dict(server=server),
                 )
+
             server = meta.add_server_interfaces(self, server)
 
         else:
@@ -1016,15 +1056,11 @@ class ComputeCloudMixin:
         # If we have boot_from_volume but no root volume, then we're
         # booting an image from volume
         if boot_volume:
-            volume = self.get_volume(boot_volume)
+            volume = self.block_storage.find_volume(boot_volume)
             if not volume:
-                raise exc.OpenStackCloudException(
-                    'Volume {boot_volume} is not a valid volume'
-                    ' in {cloud}:{region}'.format(
-                        boot_volume=boot_volume,
-                        cloud=self.name,
-                        region=self._compute_region,
-                    )
+                raise exceptions.SDKException(
+                    f"Volume {volume} was requested but was not found "
+                    f"on the cloud"
                 )
             block_mapping = {
                 'boot_index': '0',
@@ -1036,19 +1072,17 @@ class ComputeCloudMixin:
             kwargs['block_device_mapping_v2'].append(block_mapping)
             kwargs['imageRef'] = ''
         elif boot_from_volume:
+            # TODO(stephenfin): Drop support for dicts: we should only accept
+            # strings or Image objects
             if isinstance(image, dict):
                 image_obj = image
             else:
-                image_obj = self.get_image(image)
-            if not image_obj:
-                raise exc.OpenStackCloudException(
-                    'Image {image} is not a valid image in'
-                    ' {cloud}:{region}'.format(
-                        image=image,
-                        cloud=self.name,
-                        region=self._compute_region,
+                image_obj = self.image.find_image(image)
+                if not image_obj:
+                    raise exceptions.SDKException(
+                        f"Image {image} was requested but was not found "
+                        f"on the cloud"
                     )
-                )
 
             block_mapping = {
                 'boot_index': '0',
@@ -1064,23 +1098,19 @@ class ComputeCloudMixin:
             # If we're attaching volumes on boot but booting from an image,
             # we need to specify that in the BDM.
             block_mapping = {
-                u'boot_index': 0,
-                u'delete_on_termination': True,
-                u'destination_type': u'local',
-                u'source_type': u'image',
-                u'uuid': kwargs['imageRef'],
+                'boot_index': 0,
+                'delete_on_termination': True,
+                'destination_type': 'local',
+                'source_type': 'image',
+                'uuid': kwargs['imageRef'],
             }
             kwargs['block_device_mapping_v2'].append(block_mapping)
         for volume in volumes:
-            volume_obj = self.get_volume(volume)
+            volume_obj = self.block_storage.find_volume(volume)
             if not volume_obj:
-                raise exc.OpenStackCloudException(
-                    'Volume {volume} is not a valid volume'
-                    ' in {cloud}:{region}'.format(
-                        volume=volume,
-                        cloud=self.name,
-                        region=self._compute_region,
-                    )
+                raise exceptions.SDKException(
+                    f"Volume {volume} was requested but was not found "
+                    f"on the cloud"
                 )
             block_mapping = {
                 'boot_index': '-1',
@@ -1126,7 +1156,7 @@ class ComputeCloudMixin:
             # and pass it down into the IP stack.
             remaining_timeout = timeout - int(time.time() - start_time)
             if remaining_timeout <= 0:
-                raise exc.OpenStackCloudTimeout(timeout_message)
+                raise exceptions.ResourceTimeout(timeout_message)
 
             server = self.get_active_server(
                 server=server,
@@ -1159,17 +1189,17 @@ class ComputeCloudMixin:
                 and server['fault'] is not None
                 and 'message' in server['fault']
             ):
-                raise exc.OpenStackCloudException(
-                    "Error in creating the server."
-                    " Compute service reports fault: {reason}".format(
+                raise exceptions.SDKException(
+                    "Error in creating the server. "
+                    "Compute service reports fault: {reason}".format(
                         reason=server['fault']['message']
                     ),
                     extra_data=dict(server=server),
                 )
 
-            raise exc.OpenStackCloudException(
-                "Error in creating the server"
-                " (no further information available)",
+            raise exceptions.SDKException(
+                "Error in creating the server "
+                "(no further information available)",
                 extra_data=dict(server=server),
             )
 
@@ -1195,13 +1225,13 @@ class ComputeCloudMixin:
             try:
                 self._delete_server(server=server, wait=wait, timeout=timeout)
             except Exception as e:
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     'Server reached ACTIVE state without being'
                     ' allocated an IP address AND then could not'
-                    ' be deleted: {0}'.format(e),
+                    ' be deleted: {}'.format(e),
                     extra_data=dict(server=server),
                 )
-            raise exc.OpenStackCloudException(
+            raise exceptions.SDKException(
                 'Server reached ACTIVE state without being'
                 ' allocated an IP address.',
                 extra_data=dict(server=server),
@@ -1253,14 +1283,14 @@ class ComputeCloudMixin:
         :param dict metadata: A dictionary with the key=value pairs
             to set in the server instance. It only updates the key=value pairs
             provided. Existing ones will remain untouched.
+
         :returns: None
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         server = self.get_server(name_or_id, bare=True)
         if not server:
-            raise exc.OpenStackCloudException(
-                'Invalid Server {server}'.format(server=name_or_id)
-            )
+            raise exceptions.SDKException(f'Invalid Server {name_or_id}')
 
         self.compute.set_server_metadata(server=server.id, **metadata)
 
@@ -1271,14 +1301,14 @@ class ComputeCloudMixin:
             to update.
         :param metadata_keys: A list with the keys to be deleted
             from the server instance.
+
         :returns: None
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         server = self.get_server(name_or_id, bare=True)
         if not server:
-            raise exc.OpenStackCloudException(
-                'Invalid Server {server}'.format(server=name_or_id)
-            )
+            raise exceptions.SDKException(f'Invalid Server {name_or_id}')
 
         self.compute.delete_server_metadata(
             server=server.id, keys=metadata_keys
@@ -1301,9 +1331,11 @@ class ComputeCloudMixin:
             associated with the instance.
         :param int delete_ip_retry: Number of times to retry deleting
             any floating ips, should the first try be unsuccessful.
+
         :returns: True if delete succeeded, False otherwise if the
             server does not exist.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         # If delete_ips is True, we need the server to not be bare.
         server = self.compute.find_server(name_or_id, ignore_missing=True)
@@ -1332,7 +1364,7 @@ class ComputeCloudMixin:
                 ip = self.get_floating_ip(
                     id=None, filters={'floating_ip_address': fip['addr']}
                 )
-            except exc.OpenStackCloudURINotFound:
+            except exceptions.NotFoundException:
                 # We're deleting. If it doesn't exist - awesome
                 # NOTE(mordred) If the cloud is a nova FIP cloud but
                 #               floating_ip_source is set to neutron, this
@@ -1342,7 +1374,7 @@ class ComputeCloudMixin:
                 continue
             deleted = self.delete_floating_ip(ip['id'], retry=delete_ip_retry)
             if not deleted:
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     "Tried to delete floating ip {floating_ip}"
                     " associated with server {id} but there was"
                     " an error deleting it. Not deleting server.".format(
@@ -1396,8 +1428,10 @@ class ComputeCloudMixin:
             detailed = False.
         :param name: New name for the server
         :param description: New description for the server
+
         :returns: The updated compute ``Server`` object.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         server = self.compute.find_server(name_or_id, ignore_missing=False)
 
@@ -1410,8 +1444,10 @@ class ComputeCloudMixin:
 
         :param name: Name of the server group being created
         :param policies: List of policies for the server group.
+
         :returns: The created compute ``ServerGroup`` object.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         sg_attrs = {'name': name}
         if policies:
@@ -1424,8 +1460,10 @@ class ComputeCloudMixin:
         """Delete a server group.
 
         :param name_or_id: Name or ID of the server group to delete
+
         :returns: True if delete succeeded, False otherwise
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         server_group = self.get_server_group(name_or_id)
         if not server_group:
@@ -1462,8 +1500,10 @@ class ComputeCloudMixin:
         :param swap: Swap space in MB
         :param rxtx_factor: RX/TX factor
         :param is_public: Make flavor accessible to the public
+
         :returns: The created compute ``Flavor`` object.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         attrs = {
             'disk': disk,
@@ -1486,8 +1526,10 @@ class ComputeCloudMixin:
         """Delete a flavor
 
         :param name_or_id: ID or name of the flavor to delete.
+
         :returns: True if delete succeeded, False otherwise.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         try:
             flavor = self.compute.find_flavor(name_or_id)
@@ -1497,8 +1539,8 @@ class ComputeCloudMixin:
             self.compute.delete_flavor(flavor)
             return True
         except exceptions.SDKException:
-            raise exceptions.OpenStackCloudException(
-                "Unable to delete flavor {name}".format(name=name_or_id)
+            raise exceptions.SDKException(
+                f"Unable to delete flavor {name_or_id}"
             )
 
     def set_flavor_specs(self, flavor_id, extra_specs):
@@ -1507,8 +1549,10 @@ class ComputeCloudMixin:
         :param string flavor_id: ID of the flavor to update.
         :param dict extra_specs: Dictionary of key-value pairs.
 
-        :raises: OpenStackCloudException on operation error.
-        :raises: OpenStackCloudResourceNotFound if flavor ID is not found.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
+        :raises: :class:`~openstack.exceptions.BadRequestException` if flavor
+            ID is not found.
         """
         self.compute.create_flavor_extra_specs(flavor_id, extra_specs)
 
@@ -1518,8 +1562,10 @@ class ComputeCloudMixin:
         :param string flavor_id: ID of the flavor to update.
         :param keys: List of spec keys to delete.
 
-        :raises: OpenStackCloudException on operation error.
-        :raises: OpenStackCloudResourceNotFound if flavor ID is not found.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
+        :raises: :class:`~openstack.exceptions.BadRequestException` if flavor
+            ID is not found.
         """
         for key in keys:
             self.compute.delete_flavor_extra_specs_property(flavor_id, key)
@@ -1530,7 +1576,8 @@ class ComputeCloudMixin:
         :param string flavor_id: ID of the private flavor.
         :param string project_id: ID of the project/tenant.
 
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         self.compute.flavor_add_tenant_access(flavor_id, project_id)
 
@@ -1540,7 +1587,8 @@ class ComputeCloudMixin:
         :param string flavor_id: ID of the private flavor.
         :param string project_id: ID of the project/tenant.
 
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         self.compute.flavor_remove_tenant_access(flavor_id, project_id)
 
@@ -1548,8 +1596,10 @@ class ComputeCloudMixin:
         """List access from a private flavor for a project/tenant.
 
         :param string flavor_id: ID of the private flavor.
+
         :returns: List of dicts with flavor_id and tenant_id attributes.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         return self.compute.get_flavor_access(flavor_id)
 
@@ -1569,10 +1619,11 @@ class ComputeCloudMixin:
 
         :param name: aggregate name or id.
         :param filters: a dict containing additional filters to use.
+
         :returns: A list of compute ``Aggregate`` objects matching the search
             criteria.
-        :raises: ``OpenStackCloudException``: if something goes wrong during
-            the OpenStack API call.
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call.
         """
         aggregates = self.list_aggregates()
         return _utils._filter_list(aggregates, name_or_id, filters)
@@ -1612,8 +1663,10 @@ class ComputeCloudMixin:
 
         :param name: Name of the host aggregate being created
         :param availability_zone: Availability zone to assign hosts
+
         :returns: The created compute ``Aggregate`` object.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         return self.compute.create_aggregate(
             name=name, availability_zone=availability_zone
@@ -1626,8 +1679,10 @@ class ComputeCloudMixin:
         :param name_or_id: Name or ID of the aggregate being updated.
         :param name: New aggregate name
         :param availability_zone: Availability zone to assign to hosts
+
         :returns: The updated compute ``Aggregate`` object.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         aggregate = self.get_aggregate(name_or_id)
         return self.compute.update_aggregate(aggregate, **kwargs)
@@ -1636,8 +1691,10 @@ class ComputeCloudMixin:
         """Delete a host aggregate.
 
         :param name_or_id: Name or ID of the host aggregate to delete.
+
         :returns: True if delete succeeded, False otherwise.
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         if isinstance(name_or_id, (str, bytes)) and not name_or_id.isdigit():
             aggregate = self.get_aggregate(name_or_id)
@@ -1662,12 +1719,12 @@ class ComputeCloudMixin:
             {'key': None} to remove a key)
 
         :returns: a dict representing the new host aggregate.
-
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         aggregate = self.get_aggregate(name_or_id)
         if not aggregate:
-            raise exc.OpenStackCloudException(
+            raise exceptions.SDKException(
                 "Host aggregate %s not found." % name_or_id
             )
 
@@ -1679,11 +1736,12 @@ class ComputeCloudMixin:
         :param name_or_id: Name or ID of the host aggregate.
         :param host_name: Host to add.
 
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         aggregate = self.get_aggregate(name_or_id)
         if not aggregate:
-            raise exc.OpenStackCloudException(
+            raise exceptions.SDKException(
                 "Host aggregate %s not found." % name_or_id
             )
 
@@ -1695,11 +1753,12 @@ class ComputeCloudMixin:
         :param name_or_id: Name or ID of the host aggregate.
         :param host_name: Host to remove.
 
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         aggregate = self.get_aggregate(name_or_id)
         if not aggregate:
-            raise exc.OpenStackCloudException(
+            raise exceptions.SDKException(
                 "Host aggregate %s not found." % name_or_id
             )
 
@@ -1711,21 +1770,21 @@ class ComputeCloudMixin:
         :param name_or_id: project name or id
         :param kwargs: key/value pairs of quota name and quota value
 
-        :raises: OpenStackCloudException if the resource to set the
-            quota does not exist.
+        :raises: :class:`~openstack.exceptions.SDKException` if the resource to
+            set the quota does not exist.
         """
-        proj = self.identity.find_project(name_or_id, ignore_missing=False)
+        project = self.identity.find_project(name_or_id, ignore_missing=False)
         kwargs['force'] = True
-        self.compute.update_quota_set(
-            _qs.QuotaSet(project_id=proj.id), **kwargs
-        )
+        self.compute.update_quota_set(project=project, **kwargs)
 
     def get_compute_quotas(self, name_or_id):
         """Get quota for a project
 
         :param name_or_id: project name or id
+
         :returns: A compute ``QuotaSet`` object if found, else None.
-        :raises: OpenStackCloudException if it's not a valid project
+        :raises: :class:`~openstack.exceptions.SDKException` if it's not a
+            valid project
         """
         proj = self.identity.find_project(name_or_id, ignore_missing=False)
         return self.compute.get_quota_set(proj)
@@ -1734,9 +1793,9 @@ class ComputeCloudMixin:
         """Delete quota for a project
 
         :param name_or_id: project name or id
-        :raises: OpenStackCloudException if it's not a valid project or the
-            nova client call failed
-        :returns: None
+
+        :raises: :class:`~openstack.exceptions.SDKException` if it's not a
+            valid project or the nova client call failed
         """
         proj = self.identity.find_project(name_or_id, ignore_missing=False)
         self.compute.revert_quota_set(proj)
@@ -1750,9 +1809,10 @@ class ComputeCloudMixin:
             was started)
         :param end: :class:`datetime.datetime` or string. End date in UTC.
             Defaults to now
-        :raises: OpenStackCloudException if it's not a valid project
 
         :returns: A :class:`~openstack.compute.v2.usage.Usage` object
+        :raises: :class:`~openstack.exceptions.SDKException` if it's not a
+            valid project
         """
 
         def parse_date(date):
@@ -1762,7 +1822,7 @@ class ComputeCloudMixin:
                 # Yes. This is an exception mask. However,iso8601 is an
                 # implementation detail - and the error message is actually
                 # less informative.
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     "Date given, {date}, is invalid. Please pass in a date"
                     " string in ISO 8601 format -"
                     " YYYY-MM-DDTHH:MM:SS".format(date=date)
@@ -1773,10 +1833,11 @@ class ComputeCloudMixin:
         if isinstance(end, str):
             end = parse_date(end)
 
-        proj = self.get_project(name_or_id)
+        proj = self.identity.find_project(name_or_id)
         if not proj:
-            raise exc.OpenStackCloudException(
-                "project does not exist: {name}".format(name=proj.id)
+            raise exceptions.SDKException(
+                f"Project {name_or_id} was requested but was not found "
+                f"on the cloud"
             )
 
         return self.compute.get_usage(proj, start, end)
