@@ -67,7 +67,7 @@ class FloatingIPCloudMixin:
     def _nova_list_floating_ips(self):
         try:
             data = proxy._json_response(self.compute.get('/os-floating-ips'))
-        except exc.OpenStackCloudURINotFound:
+        except exceptions.NotFoundException:
             return []
         return self._get_and_munchify('floating_ips', data)
 
@@ -109,7 +109,7 @@ class FloatingIPCloudMixin:
         if self._use_neutron_floating():
             try:
                 return self._neutron_list_floating_ips(filters)
-            except exc.OpenStackCloudURINotFound as e:
+            except exceptions.NotFoundException as e:
                 # Nova-network don't support server-side floating ips
                 # filtering, so it's safer to return an empty list than
                 # to fallback to Nova which may return more results that
@@ -144,8 +144,8 @@ class FloatingIPCloudMixin:
         else:
             if filters:
                 raise ValueError(
-                    "Nova-network don't support server-side floating ips "
-                    "filtering. Use the search_floating_ips method instead"
+                    "nova-network doesn't support server-side floating IPs "
+                    "filtering. Use the 'search_floating_ips' method instead"
                 )
 
         floating_ips = self._nova_list_floating_ips()
@@ -159,7 +159,7 @@ class FloatingIPCloudMixin:
         neutron. `get_external_ipv4_floating_networks` is what you should
         almost certainly be using.
 
-        :returns: A list of floating IP pools
+        :returns: A list of floating IP pool objects
         """
         if not self._has_nova_extension('os-floating-ip-pools'):
             raise exc.OpenStackCloudUnavailableExtension(
@@ -180,14 +180,14 @@ class FloatingIPCloudMixin:
         :returns: A floating ip
             `:class:`~openstack.network.v2.floating_ip.FloatingIP`.
         """
-        error_message = "Error getting floating ip with ID {id}".format(id=id)
+        error_message = f"Error getting floating ip with ID {id}"
 
         if self._use_neutron_floating():
             fip = self.network.get_ip(id)
             return fip
         else:
             data = proxy._json_response(
-                self.compute.get('/os-floating-ips/{id}'.format(id=id)),
+                self.compute.get(f'/os-floating-ips/{id}'),
                 error_message=error_message,
             )
             return self._normalize_floating_ip(
@@ -206,8 +206,8 @@ class FloatingIPCloudMixin:
         :param server: (server) Server the Floating IP is for
 
         :returns: a list of floating IP addresses.
-        :raises: ``OpenStackCloudResourceNotFound``, if an external network
-            that meets the specified criteria cannot be found.
+        :raises: :class:`~openstack.exceptions.BadRequestException` if an
+            external network that meets the specified criteria cannot be found.
         """
         if project_id is None:
             # Make sure we are only listing floatingIPs allocated the current
@@ -229,8 +229,8 @@ class FloatingIPCloudMixin:
                     break
 
             if floating_network_id is None:
-                raise exc.OpenStackCloudResourceNotFound(
-                    "unable to find external network {net}".format(net=network)
+                raise exceptions.NotFoundException(
+                    f"unable to find external network {network}"
                 )
         else:
             floating_network_id = self._get_floating_network_id()
@@ -265,18 +265,17 @@ class FloatingIPCloudMixin:
         :param pool: Nova floating IP pool name.
 
         :returns: a list of floating IP addresses.
-
-        :raises: ``OpenStackCloudResourceNotFound``, if a floating IP pool
-            is not specified and cannot be found.
+        :raises: :class:`~openstack.exceptions.BadRequestException` if a
+            floating IP pool is not specified and cannot be found.
         """
 
         with _utils.openstacksdk_exceptions(
-            "Unable to create floating IP in pool {pool}".format(pool=pool)
+            f"Unable to create floating IP in pool {pool}"
         ):
             if pool is None:
                 pools = self.list_floating_ip_pools()
                 if not pools:
-                    raise exc.OpenStackCloudResourceNotFound(
+                    raise exceptions.NotFoundException(
                         "unable to find a floating ip pool"
                     )
                 pool = pools[0]['name']
@@ -298,13 +297,14 @@ class FloatingIPCloudMixin:
 
     def _find_floating_network_by_router(self):
         """Find the network providing floating ips by looking at routers."""
-        for router in self.list_routers():
+        for router in self.network.routers():
             if router['admin_state_up']:
                 network_id = router.get('external_gateway_info', {}).get(
                     'network_id'
                 )
                 if network_id:
-                    return network_id
+                    self._floating_network_by_router = network_id
+        return self._floating_network_by_router
 
     def available_floating_ip(self, network=None, server=None):
         """Get a floating IP from a network or a pool.
@@ -323,7 +323,7 @@ class FloatingIPCloudMixin:
                     network=network, server=server
                 )
                 return f_ips[0]
-            except exc.OpenStackCloudURINotFound as e:
+            except exceptions.NotFoundException as e:
                 self.log.debug(
                     "Something went wrong talking to neutron API: "
                     "'%(msg)s'. Trying with Nova.",
@@ -346,7 +346,7 @@ class FloatingIPCloudMixin:
             if floating_network:
                 floating_network_id = floating_network
             else:
-                raise exc.OpenStackCloudResourceNotFound(
+                raise exceptions.NotFoundException(
                     "unable to find an external network"
                 )
         return floating_network_id
@@ -384,8 +384,8 @@ class FloatingIPCloudMixin:
                         provided.
 
         :returns: a floating IP address
-
-        :raises: ``OpenStackCloudException``, on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         if self._use_neutron_floating():
             try:
@@ -398,7 +398,7 @@ class FloatingIPCloudMixin:
                     wait=wait,
                     timeout=timeout,
                 )
-            except exc.OpenStackCloudURINotFound as e:
+            except exceptions.NotFoundException as e:
                 self.log.debug(
                     "Something went wrong talking to neutron API: "
                     "'%(msg)s'. Trying with Nova.",
@@ -407,12 +407,12 @@ class FloatingIPCloudMixin:
                 # Fall-through, trying with Nova
 
         if port:
-            raise exc.OpenStackCloudException(
-                "This cloud uses nova-network which does not support"
-                " arbitrary floating-ip/port mappings. Please nudge"
-                " your cloud provider to upgrade the networking stack"
-                " to neutron, or alternately provide the server,"
-                " fixed_address and nat_destination arguments as appropriate"
+            raise exceptions.SDKException(
+                "This cloud uses nova-network which does not support "
+                "arbitrary floating-ip/port mappings. Please nudge "
+                "your cloud provider to upgrade the networking stack "
+                "to neutron, or alternately provide the server, "
+                "fixed_address and nat_destination arguments as appropriate"
             )
         # Else, we are using Nova network
         f_ips = self._normalize_floating_ips(
@@ -440,9 +440,9 @@ class FloatingIPCloudMixin:
                 try:
                     network = self.network.find_network(network_name_or_id)
                 except exceptions.ResourceNotFound:
-                    raise exc.OpenStackCloudResourceNotFound(
+                    raise exceptions.NotFoundException(
                         "unable to find network for floating ips with ID "
-                        "{0}".format(network_name_or_id)
+                        "{}".format(network_name_or_id)
                     )
                 network_id = network['id']
             else:
@@ -481,10 +481,10 @@ class FloatingIPCloudMixin:
                         fip = self.get_floating_ip(fip_id)
                         if fip and fip['status'] == 'ACTIVE':
                             break
-                except exc.OpenStackCloudTimeout:
+                except exceptions.ResourceTimeout:
                     self.log.error(
-                        "Timed out on floating ip %(fip)s becoming active."
-                        " Deleting",
+                        "Timed out on floating ip %(fip)s becoming active. "
+                        "Deleting",
                         {'fip': fip_id},
                     )
                     try:
@@ -499,29 +499,29 @@ class FloatingIPCloudMixin:
                     raise
             if fip['port_id'] != port:
                 if server:
-                    raise exc.OpenStackCloudException(
-                        "Attempted to create FIP on port {port} for server"
-                        " {server} but FIP has port {port_id}".format(
+                    raise exceptions.SDKException(
+                        "Attempted to create FIP on port {port} for server "
+                        "{server} but FIP has port {port_id}".format(
                             port=port,
                             port_id=fip['port_id'],
                             server=server['id'],
                         )
                     )
                 else:
-                    raise exc.OpenStackCloudException(
-                        "Attempted to create FIP on port {port}"
-                        " but something went wrong".format(port=port)
+                    raise exceptions.SDKException(
+                        "Attempted to create FIP on port {port} "
+                        "but something went wrong".format(port=port)
                     )
         return fip
 
     def _nova_create_floating_ip(self, pool=None):
         with _utils.openstacksdk_exceptions(
-            "Unable to create floating IP in pool {pool}".format(pool=pool)
+            f"Unable to create floating IP in pool {pool}"
         ):
             if pool is None:
                 pools = self.list_floating_ip_pools()
                 if not pools:
-                    raise exc.OpenStackCloudResourceNotFound(
+                    raise exceptions.NotFoundException(
                         "unable to find a floating ip pool"
                     )
                 pool = pools[0]['name']
@@ -543,14 +543,14 @@ class FloatingIPCloudMixin:
 
         :param floating_ip_id: a floating IP address ID.
         :param retry: number of times to retry. Optional, defaults to 1,
-                      which is in addition to the initial delete call.
-                      A value of 0 will also cause no checking of results to
-                      occur.
+            which is in addition to the initial delete call.
+            A value of 0 will also cause no checking of results to
+            occur.
 
         :returns: True if the IP address has been deleted, False if the IP
-                  address was not found.
-
-        :raises: ``OpenStackCloudException``, on operation error.
+            address was not found.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         for count in range(0, max(0, retry) + 1):
             result = self._delete_floating_ip(floating_ip_id)
@@ -566,11 +566,11 @@ class FloatingIPCloudMixin:
             if not f_ip or f_ip['status'] == 'DOWN':
                 return True
 
-        raise exc.OpenStackCloudException(
-            "Attempted to delete Floating IP {ip} with ID {id} a total of"
-            " {retry} times. Although the cloud did not indicate any errors"
-            " the floating ip is still in existence. Aborting further"
-            " operations.".format(
+        raise exceptions.SDKException(
+            "Attempted to delete Floating IP {ip} with ID {id} a total of "
+            "{retry} times. Although the cloud did not indicate any errors "
+            "the floating IP is still in existence. Aborting further "
+            "operations.".format(
                 id=floating_ip_id,
                 ip=f_ip['floating_ip_address'],
                 retry=retry + 1,
@@ -581,7 +581,7 @@ class FloatingIPCloudMixin:
         if self._use_neutron_floating():
             try:
                 return self._neutron_delete_floating_ip(floating_ip_id)
-            except exc.OpenStackCloudURINotFound as e:
+            except exceptions.NotFoundException as e:
                 self.log.debug(
                     "Something went wrong talking to neutron API: "
                     "'%(msg)s'. Trying with Nova.",
@@ -599,14 +599,12 @@ class FloatingIPCloudMixin:
     def _nova_delete_floating_ip(self, floating_ip_id):
         try:
             proxy._json_response(
-                self.compute.delete(
-                    '/os-floating-ips/{id}'.format(id=floating_ip_id)
-                ),
+                self.compute.delete(f'/os-floating-ips/{floating_ip_id}'),
                 error_message='Unable to delete floating IP {fip_id}'.format(
                     fip_id=floating_ip_id
                 ),
             )
-        except exc.OpenStackCloudURINotFound:
+        except exceptions.NotFoundException:
             return False
         return True
 
@@ -624,13 +622,12 @@ class FloatingIPCloudMixin:
         IPs by passing in a server to the create_floating_ip call.
 
         :param retry: number of times to retry. Optional, defaults to 1,
-                      which is in addition to the initial delete call.
-                      A value of 0 will also cause no checking of results to
-                      occur.
+            which is in addition to the initial delete call.
+            A value of 0 will also cause no checking of results to occur.
 
         :returns: Number of Floating IPs deleted, False if none
-
-        :raises: ``OpenStackCloudException``, on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         processed = []
         if self._use_neutron_floating():
@@ -658,19 +655,19 @@ class FloatingIPCloudMixin:
         :param server: Server dict
         :param floating_ip: Floating IP dict to attach
         :param fixed_address: (optional) fixed address to which attach the
-                              floating IP to.
+            floating IP to.
         :param wait: (optional) Wait for the address to appear as assigned
-                     to the server. Defaults to False.
+            to the server. Defaults to False.
         :param timeout: (optional) Seconds to wait, defaults to 60.
-                        See the ``wait`` parameter.
+            See the ``wait`` parameter.
         :param skip_attach: (optional) Skip the actual attach and just do
-                            the wait. Defaults to False.
+            the wait. Defaults to False.
         :param nat_destination: The fixed network the server's port for the
-                                FIP to attach to will come from.
+            FIP to attach to will come from.
 
         :returns: The server ``openstack.compute.v2.server.Server``
-
-        :raises: OpenStackCloudException, on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         # Short circuit if we're asking to attach an IP that's already
         # attached
@@ -681,7 +678,7 @@ class FloatingIPCloudMixin:
             # old to check whether it belongs to us now, thus refresh
             # the server data and try again. There are some clouds, which
             # explicitely forbids FIP assign call if it is already assigned.
-            server = self.get_server_by_id(server['id'])
+            server = self.compute.get_server(server['id'])
             ext_ip = meta.get_server_ip(
                 server, ext_tag='floating', public=True
             )
@@ -697,7 +694,7 @@ class FloatingIPCloudMixin:
                         fixed_address=fixed_address,
                         nat_destination=nat_destination,
                     )
-                except exc.OpenStackCloudURINotFound as e:
+                except exceptions.NotFoundException as e:
                     self.log.debug(
                         "Something went wrong talking to neutron API: "
                         "'%(msg)s'. Trying with Nova.",
@@ -720,7 +717,7 @@ class FloatingIPCloudMixin:
                 "Timeout waiting for the floating IP to be attached.",
                 wait=min(5, timeout),
             ):
-                server = self.get_server_by_id(server_id)
+                server = self.compute.get_server(server_id)
                 ext_ip = meta.get_server_ip(
                     server, ext_tag='floating', public=True
                 )
@@ -738,8 +735,8 @@ class FloatingIPCloudMixin:
             nat_destination=nat_destination,
         )
         if not port:
-            raise exc.OpenStackCloudException(
-                "unable to find a port for server {0}".format(server['id'])
+            raise exceptions.SDKException(
+                "unable to find a port for server {}".format(server['id'])
             )
 
         floating_ip_args = {'port_id': port['id']}
@@ -753,8 +750,8 @@ class FloatingIPCloudMixin:
     ):
         f_ip = self.get_floating_ip(id=floating_ip_id)
         if f_ip is None:
-            raise exc.OpenStackCloudException(
-                "unable to find floating IP {0}".format(floating_ip_id)
+            raise exceptions.SDKException(
+                f"unable to find floating IP {floating_ip_id}"
             )
         error_message = "Error attaching IP {ip} to instance {id}".format(
             ip=floating_ip_id, id=server_id
@@ -764,7 +761,7 @@ class FloatingIPCloudMixin:
             body['fixed_address'] = fixed_address
         return proxy._json_response(
             self.compute.post(
-                '/servers/{server_id}/action'.format(server_id=server_id),
+                f'/servers/{server_id}/action',
                 json=dict(addFloatingIp=body),
             ),
             error_message=error_message,
@@ -777,16 +774,16 @@ class FloatingIPCloudMixin:
         :param floating_ip_id: Id of the floating IP to detach.
 
         :returns: True if the IP has been detached, or False if the IP wasn't
-                  attached to any server.
-
-        :raises: ``OpenStackCloudException``, on operation error.
+            attached to any server.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         if self._use_neutron_floating():
             try:
                 return self._neutron_detach_ip_from_server(
                     server_id=server_id, floating_ip_id=floating_ip_id
                 )
-            except exc.OpenStackCloudURINotFound as e:
+            except exceptions.NotFoundException as e:
                 self.log.debug(
                     "Something went wrong talking to neutron API: "
                     "'%(msg)s'. Trying with Nova.",
@@ -807,11 +804,9 @@ class FloatingIPCloudMixin:
             self.network.update_ip(floating_ip_id, port_id=None)
         except exceptions.SDKException:
             raise exceptions.SDKException(
-                (
-                    "Error detaching IP {ip} from "
-                    "server {server_id}".format(
-                        ip=floating_ip_id, server_id=server_id
-                    )
+                "Error detaching IP {ip} from "
+                "server {server_id}".format(
+                    ip=floating_ip_id, server_id=server_id
                 )
             )
 
@@ -820,15 +815,15 @@ class FloatingIPCloudMixin:
     def _nova_detach_ip_from_server(self, server_id, floating_ip_id):
         f_ip = self.get_floating_ip(id=floating_ip_id)
         if f_ip is None:
-            raise exc.OpenStackCloudException(
-                "unable to find floating IP {0}".format(floating_ip_id)
+            raise exceptions.SDKException(
+                f"unable to find floating IP {floating_ip_id}"
             )
         error_message = "Error detaching IP {ip} from instance {id}".format(
             ip=floating_ip_id, id=server_id
         )
         return proxy._json_response(
             self.compute.post(
-                '/servers/{server_id}/action'.format(server_id=server_id),
+                f'/servers/{server_id}/action',
                 json=dict(
                     removeFloatingIp=dict(address=f_ip['floating_ip_address'])
                 ),
@@ -860,11 +855,11 @@ class FloatingIPCloudMixin:
         :param fixed_address: a fixed address
         :param reuse: Try to reuse existing ips. Defaults to True.
         :param wait: (optional) Wait for the address to appear as assigned
-                     to the server. Defaults to False.
+            to the server. Defaults to False.
         :param timeout: (optional) Seconds to wait, defaults to 60.
-                        See the ``wait`` parameter.
+            See the ``wait`` parameter.
         :param nat_destination: (optional) the name of the network of the
-                                port to associate with the floating ip.
+            port to associate with the floating ip.
 
         :returns: the updated server ``openstack.compute.v2.server.Server``
         """
@@ -881,7 +876,7 @@ class FloatingIPCloudMixin:
                 timeout=timeout,
             )
             timeout = timeout - (time.time() - start_time)
-            server = self.get_server(server.id)
+            server = self.compute.get_server(server.id)
 
         # We run attach as a second call rather than in the create call
         # because there are code flows where we will not have an attached
@@ -911,21 +906,20 @@ class FloatingIPCloudMixin:
         :param server: a server object
         :param ips: list of floating IP addresses or a single address
         :param wait: (optional) Wait for the address to appear as assigned
-                     to the server. Defaults to False.
+            to the server. Defaults to False.
         :param timeout: (optional) Seconds to wait, defaults to 60.
-                        See the ``wait`` parameter.
+            See the ``wait`` parameter.
         :param fixed_address: (optional) Fixed address of the server to
-                                         attach the IP to
+            attach the IP to
         :param nat_destination: (optional) Name or ID of the network that
-                                          the fixed IP to attach the
-                                          floating IP should be on
+            the fixed IP to attach the floating IP should be on
 
         :returns: The updated server ``openstack.compute.v2.server.Server``
-
-        :raises: ``OpenStackCloudException``, on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
 
-        if type(ips) != list:
+        if type(ips) is list:
             ips = [ips]
 
         for ip in ips:
@@ -954,15 +948,14 @@ class FloatingIPCloudMixin:
 
         :param server: a server dictionary.
         :param reuse: Whether or not to attempt to reuse IPs, defaults
-                      to True.
+            to True.
         :param wait: (optional) Wait for the address to appear as assigned
-                     to the server. Defaults to False.
+            to the server. Defaults to False.
         :param timeout: (optional) Seconds to wait, defaults to 60.
-                        See the ``wait`` parameter.
+            See the ``wait`` parameter.
         :param reuse: Try to reuse existing ips. Defaults to True.
 
         :returns: Floating IP address attached to server.
-
         """
         server = self._add_auto_ip(
             server, wait=wait, timeout=timeout, reuse=reuse
@@ -999,16 +992,16 @@ class FloatingIPCloudMixin:
                 timeout=timeout,
                 skip_attach=skip_attach,
             )
-        except exc.OpenStackCloudTimeout:
+        except exceptions.ResourceTimeout:
             if self._use_neutron_floating() and created:
                 # We are here because we created an IP on the port
                 # It failed. Delete so as not to leak an unmanaged
                 # resource
                 self.log.error(
-                    "Timeout waiting for floating IP to become"
-                    " active. Floating IP %(ip)s:%(id)s was created for"
-                    " server %(server)s but is being deleted due to"
-                    " activation failure.",
+                    "Timeout waiting for floating IP to become "
+                    "active. Floating IP %(ip)s:%(id)s was created for "
+                    "server %(server)s but is being deleted due to "
+                    "activation failure.",
                     {
                         'ip': f_ip['floating_ip_address'],
                         'id': f_ip['id'],
@@ -1131,7 +1124,7 @@ class FloatingIPCloudMixin:
         # No floating ip network - no FIPs
         try:
             self._get_floating_network_id()
-        except exc.OpenStackCloudException:
+        except exceptions.SDKException:
             return False
 
         (port_obj, fixed_ip_address) = self._nat_destination_port(
@@ -1158,8 +1151,7 @@ class FloatingIPCloudMixin:
         :param fixed_address: Fixed ip address of the port
         :param nat_destination: Name or ID of the network of the port.
         """
-        port_filter = {'device_id': server['id']}
-        ports = self.search_ports(filters=port_filter)
+        ports = list(self.network.ports(device_id=server['id']))
         if not ports:
             return (None, None)
 
@@ -1167,9 +1159,9 @@ class FloatingIPCloudMixin:
         if not fixed_address:
             if len(ports) > 1:
                 if nat_destination:
-                    nat_network = self.get_network(nat_destination)
+                    nat_network = self.network.find_network(nat_destination)
                     if not nat_network:
-                        raise exc.OpenStackCloudException(
+                        raise exceptions.SDKException(
                             'NAT Destination {nat_destination} was configured'
                             ' but not found on the cloud. Please check your'
                             ' config and your cloud and try again.'.format(
@@ -1180,7 +1172,7 @@ class FloatingIPCloudMixin:
                     nat_network = self.get_nat_destination()
 
                 if not nat_network:
-                    raise exc.OpenStackCloudException(
+                    raise exceptions.SDKException(
                         'Multiple ports were found for server {server}'
                         ' but none of the networks are a valid NAT'
                         ' destination, so it is impossible to add a'
@@ -1198,7 +1190,7 @@ class FloatingIPCloudMixin:
                     if maybe_port['network_id'] == nat_network['id']:
                         maybe_ports.append(maybe_port)
                 if not maybe_ports:
-                    raise exc.OpenStackCloudException(
+                    raise exceptions.SDKException(
                         'No port on server {server} was found matching'
                         ' your NAT destination network {dest}. Please '
                         ' check your config'.format(
@@ -1224,9 +1216,9 @@ class FloatingIPCloudMixin:
                     if ip.version == 4:
                         fixed_address = address['ip_address']
                         return port, fixed_address
-            raise exc.OpenStackCloudException(
+            raise exceptions.SDKException(
                 "unable to find a free fixed IPv4 address for server "
-                "{0}".format(server['id'])
+                "{}".format(server['id'])
             )
         # unfortunately a port can have more than one fixed IP:
         # we can't use the search_ports filtering for fixed_address as
@@ -1266,7 +1258,7 @@ class FloatingIPCloudMixin:
         Neutron.
         This function extract attributes that are common to Nova and Neutron
         floating IP resource.
-        If the whole structure is needed inside shade, shade provides private
+        If the whole structure is needed inside openstacksdk there are private
         methods that returns "original" objects (e.g.
         _neutron_allocate_floating_ip)
 
