@@ -12,12 +12,16 @@
 
 import os
 import time
+import typing as ty
 import warnings
 
 from openstack import exceptions
+from openstack.image.v2 import cache as _cache
 from openstack.image.v2 import image as _image
 from openstack.image.v2 import member as _member
 from openstack.image.v2 import metadef_namespace as _metadef_namespace
+from openstack.image.v2 import metadef_object as _metadef_object
+from openstack.image.v2 import metadef_property as _metadef_property
 from openstack.image.v2 import metadef_resource_type as _metadef_resource_type
 from openstack.image.v2 import metadef_schema as _metadef_schema
 from openstack.image.v2 import schema as _schema
@@ -26,6 +30,7 @@ from openstack.image.v2 import task as _task
 from openstack import proxy
 from openstack import resource
 from openstack import utils
+from openstack import warnings as os_warnings
 
 # Rackspace returns this for intermittent import errors
 _IMAGE_ERROR_396 = "Image cannot be imported. Error code: '396'"
@@ -48,11 +53,13 @@ def _get_name_and_filename(name, image_format):
 
 
 class Proxy(proxy.Proxy):
-
     _resource_registry = {
+        "cache": _cache.Cache,
         "image": _image.Image,
         "image_member": _member.Member,
         "metadef_namespace": _metadef_namespace.MetadefNamespace,
+        "metadef_resource_type": _metadef_resource_type.MetadefResourceType,
+        "metadef_resource_type_association": _metadef_resource_type.MetadefResourceTypeAssociation,  # noqa
         "schema": _schema.Schema,
         "info_import": _si.Import,
         "info_store": _si.Store,
@@ -72,7 +79,39 @@ class Proxy(proxy.Proxy):
     _SHADE_IMAGE_SHA256_KEY = 'owner_specified.shade.sha256'
     _SHADE_IMAGE_OBJECT_KEY = 'owner_specified.shade.object'
 
+    # ====== CACHE MANAGEMENT======
+    def get_image_cache(self):
+        return self._get(_cache.Cache, requires_id=False)
+
+    def cache_delete_image(self, image, ignore_missing=True):
+        """Delete an image from cache.
+
+        :param image: The value can be either the name of an image or a
+            :class:`~openstack.image.v2.image.Image`
+            instance.
+        :param bool ignore_missing: When set to ``False``,
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the metadef namespace does not exist.
+        :returns: ``None``
+        """
+        return self._delete(_cache.Cache, image, ignore_missing=ignore_missing)
+
+    def queue_image(self, image_id):
+        """Queue image(s) for caching."""
+        cache = self._get_resource(_cache.Cache, None)
+        return cache.queue(self, image_id)
+
+    def clear_cache(self, target='both'):
+        """Clear all images from cache, queue or both
+
+        :param target: Specify which target you want to clear
+            One of: ``both``(default), ``cache``, ``queue``.
+        """
+        cache = self._get_resource(_cache.Cache, None)
+        return cache.clear(self, target)
+
     # ====== IMAGES ======
+
     def create_image(
         self,
         name,
@@ -294,16 +333,16 @@ class Proxy(proxy.Proxy):
                 validate_checksum=validate_checksum,
                 use_import=use_import,
                 stores=stores,
-                all_stores=stores,
-                all_stores_must_succeed=stores,
+                all_stores=all_stores,
+                all_stores_must_succeed=all_stores_must_succeed,
                 **image_kwargs,
             )
         else:
+            properties = image_kwargs.pop('properties', {})
+            image_kwargs.update(self._make_v2_image_params(meta, properties))
             image_kwargs['name'] = name
             image = self._create(_image.Image, **image_kwargs)
 
-        self._connection._get_cache(None).invalidate()
-
         return image
 
     def import_image(
@@ -465,7 +504,10 @@ class Proxy(proxy.Proxy):
         :returns: The results of image creation
         :rtype: :class:`~openstack.image.v2.image.Image`
         """
-        warnings.warn("upload_image is deprecated. Use create_image instead.")
+        warnings.warn(
+            "upload_image is deprecated. Use create_image instead.",
+            os_warnings.OpenStackDeprecationWarning,
+        )
         # container_format and disk_format are required to be set
         # on the image by the time upload_image is called, but they're not
         # required by the _create call. Enforce them here so that we don't
@@ -556,12 +598,10 @@ class Proxy(proxy.Proxy):
             self.log.debug("Image creation failed", exc_info=True)
             raise
         except Exception as e:
-            raise exceptions.SDKException(
-                "Image creation failed: {message}".format(message=str(e))
-            )
+            raise exceptions.SDKException(f"Image creation failed: {str(e)}")
 
     def _make_v2_image_params(self, meta, properties):
-        ret = {}
+        ret: ty.Dict = {}
         for k, v in iter(properties.items()):
             if k in _INT_PROPERTIES:
                 ret[k] = int(v)
@@ -692,7 +732,6 @@ class Proxy(proxy.Proxy):
         }
 
         glance_task = self.create_task(**task_args)
-        self._connection.list_images.invalidate(self)
         if wait:
             start = time.time()
 
@@ -729,7 +768,6 @@ class Proxy(proxy.Proxy):
                 # Clean up after ourselves. The object we created is not
                 # needed after the import is done.
                 self._connection.delete_object(container, name)
-                self._connection.list_images.invalidate(self)
             return image
         else:
             return glance_task
@@ -743,7 +781,7 @@ class Proxy(proxy.Proxy):
         *,
         stream=False,
         output=None,
-        chunk_size=1024,
+        chunk_size=1024 * 1024,
     ):
         """Download an image
 
@@ -767,7 +805,7 @@ class Proxy(proxy.Proxy):
             When ``False``, return the entire contents of the response.
         :param output: Either a file object or a path to store data into.
         :param int chunk_size: size in bytes to read from the wire and buffer
-            at one time. Defaults to 1024
+            at one time. Defaults to 1024 * 1024 = 1 MiB
 
         :returns: When output is not given - the bytes comprising the given
             Image when stream is False, otherwise a :class:`requests.Response`
@@ -909,7 +947,7 @@ class Proxy(proxy.Proxy):
         for k, v in iter(kwargs.items()):
             if v and k in ['ramdisk', 'kernel']:
                 v = self._connection.get_image_id(v)
-                k = '{0}_id'.format(k)
+                k = f'{k}_id'
             properties[k] = v
 
         img_props = image.properties.copy()
@@ -922,8 +960,6 @@ class Proxy(proxy.Proxy):
 
         self.update_image(image, **img_props)
 
-        self._connection.list_images.invalidate(self._connection)
-
         return True
 
     def add_tag(self, image, tag):
@@ -963,6 +999,10 @@ class Proxy(proxy.Proxy):
             a :class:`~openstack.image.v2.member.Member`,
             comprised of the properties on the Member class.
 
+        See `Image Sharing Reference
+        <https://docs.openstack.org/api-ref/image/v2/index.html?expanded=create-image-member-detail#create-image-member>`__
+        for details.
+
         :returns: The results of member creation
         :rtype: :class:`~openstack.image.v2.member.Member`
         """
@@ -1059,6 +1099,10 @@ class Proxy(proxy.Proxy):
         :param attrs: The attributes to update on the member represented
             by ``member``.
 
+        See `Image Sharing Reference
+        <https://docs.openstack.org/api-ref/image/v2/index.html?expanded=update-image-member-detail#update-image-member>`__
+        for details.
+
         :returns: The updated member
         :rtype: :class:`~openstack.image.v2.member.Member`
         """
@@ -1156,6 +1200,134 @@ class Proxy(proxy.Proxy):
             **attrs,
         )
 
+    # ====== METADEF OBJECT ======
+    def create_metadef_object(self, namespace, **attrs):
+        """Create a new object from namespace
+
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.image.v2.metadef_object.MetadefObject`,
+            comprised of the properties on the Metadef object class.
+
+        :returns: A metadef namespace
+        :rtype: :class:`~openstack.image.v2.metadef_object.MetadefObject`
+        """
+        namespace_name = resource.Resource._get_id(namespace)
+        return self._create(
+            _metadef_object.MetadefObject,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def get_metadef_object(self, metadef_object, namespace):
+        """Get a single metadef object
+
+        :param metadef_object: The value can be the ID of a metadef_object
+            or a
+            :class:`~openstack.image.v2.metadef_object.MetadefObject`
+            instance.
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        object_name = resource.Resource._get_id(metadef_object)
+        namespace_name = resource.Resource._get_id(namespace)
+        return self._get(
+            _metadef_object.MetadefObject,
+            namespace_name=namespace_name,
+            name=object_name,
+        )
+
+    def metadef_objects(self, namespace):
+        """Get metadef object list of the namespace
+
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+
+        :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(namespace)
+        return self._list(
+            _metadef_object.MetadefObject,
+            namespace_name=namespace_name,
+        )
+
+    def update_metadef_object(self, metadef_object, namespace, **attrs):
+        """Update a single metadef object
+
+        :param metadef_object: The value can be the ID of a metadef_object or a
+            :class:`~openstack.image.v2.metadef_object.MetadefObject` instance.
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param dict attrs: Keyword arguments which will be used to update
+            a :class:`~openstack.image.v2.metadef_object.MetadefObject`
+
+        :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(namespace)
+        metadef_object = resource.Resource._get_id(metadef_object)
+        return self._update(
+            _metadef_object.MetadefObject,
+            metadef_object,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def delete_metadef_object(self, metadef_object, namespace, **attrs):
+        """Removes a single metadef object
+
+        :param metadef_object: The value can be the ID of a metadef_object or a
+            :class:`~openstack.image.v2.metadef_object.MetadefObject` instance.
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param dict attrs: Keyword arguments which will be used to update
+            a :class:`~openstack.image.v2.metadef_object.MetadefObject`
+
+        :returns: ``None``
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(namespace)
+        return self._delete(
+            _metadef_object.MetadefObject,
+            metadef_object,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def delete_all_metadef_objects(self, namespace):
+        """Delete all objects
+
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :returns: ``None``
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, namespace
+        )
+        return namespace.delete_all_objects(self)
+
     # ====== METADEF RESOURCE TYPES ======
     def metadef_resource_types(self, **query):
         """Return a generator of metadef resource types
@@ -1164,7 +1336,7 @@ class Proxy(proxy.Proxy):
         :rtype:
             :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceType`
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                when no resource can be found.
+            when no resource can be found.
         """
         return self._list(_metadef_resource_type.MetadefResourceType, **query)
 
@@ -1242,6 +1414,149 @@ class Proxy(proxy.Proxy):
             **query,
         )
 
+    # ====== METADEF PROPERTY ======
+    def create_metadef_property(self, metadef_namespace, **attrs):
+        """Create a metadef property
+
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_property.MetadefNamespace`
+            instance
+        :param attrs: The attributes to create on the metadef property
+            represented by ``metadef_property``.
+
+        :returns: The created metadef property
+        :rtype: :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        return self._create(
+            _metadef_property.MetadefProperty,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def update_metadef_property(
+        self, metadef_property, metadef_namespace, **attrs
+    ):
+        """Update a metadef property
+
+        :param metadef_property: The value can be either the name of metadef
+            property or an
+            :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+            instance.
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+        :param attrs: The attributes to update on the metadef property
+            represented by ``metadef_property``.
+
+        :returns: The updated metadef property
+        :rtype: :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        metadef_property = resource.Resource._get_id(metadef_property)
+        return self._update(
+            _metadef_property.MetadefProperty,
+            metadef_property,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def delete_metadef_property(
+        self, metadef_property, metadef_namespace, ignore_missing=True
+    ):
+        """Delete a metadef property
+
+        :param metadef_property: The value can be either the name of metadef
+            property or an
+            :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+            instance
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+        :param bool ignore_missing: When set to
+            ``False`` :class:`~openstack.exceptions.ResourceNotFound` will be
+            raised when the instance does not exist. When set to ``True``,
+            no exception will be set when attempting to delete a nonexistent
+            instance.
+
+        :returns: ``None``
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        metadef_property = resource.Resource._get_id(metadef_property)
+        return self._delete(
+            _metadef_property.MetadefProperty,
+            metadef_property,
+            namespace_name=namespace_name,
+            ignore_missing=ignore_missing,
+        )
+
+    def metadef_properties(self, metadef_namespace, **query):
+        """Return a generator of metadef properties
+
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
+
+        :returns: A generator of property objects
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        return self._list(
+            _metadef_property.MetadefProperty,
+            requires_id=False,
+            namespace_name=namespace_name,
+            **query,
+        )
+
+    def get_metadef_property(
+        self, metadef_property, metadef_namespace, **query
+    ):
+        """Get a single metadef property
+
+        :param metadef_property: The value can be either the name of metadef
+            property or an
+            :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+            instance.
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+
+        :returns: One
+            :class:`~~openstack.image.v2.metadef_property.MetadefProperty`
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        return self._get(
+            _metadef_property.MetadefProperty,
+            metadef_property,
+            namespace_name=namespace_name,
+            **query,
+        )
+
+    def delete_all_metadef_properties(self, metadef_namespace):
+        """Delete all metadata definitions property inside a specific namespace.
+
+        :param metadef_namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+
+        :returns: ``None``
+        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+            resource can be found.
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, metadef_namespace
+        )
+        return namespace.delete_all_properties(self)
+
     # ====== SCHEMAS ======
     def get_images_schema(self):
         """Get images schema
@@ -1523,7 +1838,7 @@ class Proxy(proxy.Proxy):
         if task.status.lower() == status.lower():
             return task
 
-        name = "{res}:{id}".format(res=task.__class__.__name__, id=task.id)
+        name = f"{task.__class__.__name__}:{task.id}"
         msg = "Timeout waiting for {name} to transition to {status}".format(
             name=name, status=status
         )
@@ -1572,7 +1887,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.image.v2.service_info.Store`
         """
         if details:
-            query['base_path'] = utils.urljoin(_si.Store, 'details')
+            query['base_path'] = utils.urljoin(_si.Store.base_path, 'detail')
         return self._list(_si.Store, **query)
 
     # ====== IMPORTS ======
@@ -1600,3 +1915,32 @@ class Proxy(proxy.Proxy):
             to delete failed to occur in the specified seconds.
         """
         return resource.wait_for_delete(self, res, interval, wait)
+
+    def _get_cleanup_dependencies(self):
+        return {'image': {'before': ['identity']}}
+
+    def _service_cleanup(
+        self,
+        dry_run=True,
+        client_status_queue=None,
+        identified_resources=None,
+        filters=None,
+        resource_evaluation_fn=None,
+        skip_resources=None,
+    ):
+        if self.should_skip_resource_cleanup("image", skip_resources):
+            return
+
+        project_id = self.get_project_id()
+
+        # Note that images cannot be deleted when they are still being used
+        for obj in self.images(owner=project_id):
+            self._service_cleanup_del_res(
+                self.delete_image,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn,
+            )
