@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import base64
 from unittest import mock
 
 from keystoneauth1 import adapter
@@ -23,11 +24,14 @@ from openstack import utils
 
 # NOTE: Sample data from api-ref doc
 FAKE = {
+    "automated_clean": False,
     "boot_mode": "uefi",
     "chassis_uuid": "1",  # NOTE: missed in api-ref sample
     "clean_step": {},
+    "conductor_group": None,
     "console_enabled": False,
     "created_at": "2016-08-18T22:28:48.643434+00:00",
+    "description": "A node.",
     "driver": "agent_ipmitool",
     "driver_info": {"ipmi_password": "******", "ipmi_username": "ADMIN"},
     "driver_internal_info": {},
@@ -38,6 +42,7 @@ FAKE = {
     "instance_info": {},
     "instance_uuid": None,
     "last_error": None,
+    "lessee": None,
     "links": [
         {"href": "http://127.0.0.1:6385/v1/nodes/<NODE_ID>", "rel": "self"},
         {"href": "http://127.0.0.1:6385/nodes/<NODE_ID>", "rel": "bookmark"},
@@ -118,11 +123,15 @@ class TestNode(base.TestCase):
 
         self.assertEqual(FAKE['uuid'], sot.id)
         self.assertEqual(FAKE['name'], sot.name)
-
+        self.assertEqual(
+            FAKE['automated_clean'], sot.is_automated_clean_enabled
+        )
         self.assertEqual(FAKE['boot_mode'], sot.boot_mode)
         self.assertEqual(FAKE['chassis_uuid'], sot.chassis_id)
         self.assertEqual(FAKE['clean_step'], sot.clean_step)
+        self.assertEqual(FAKE['conductor_group'], sot.conductor_group)
         self.assertEqual(FAKE['created_at'], sot.created_at)
+        self.assertEqual(FAKE['description'], sot.description)
         self.assertEqual(FAKE['driver'], sot.driver)
         self.assertEqual(FAKE['driver_info'], sot.driver_info)
         self.assertEqual(
@@ -135,6 +144,7 @@ class TestNode(base.TestCase):
         self.assertEqual(FAKE['console_enabled'], sot.is_console_enabled)
         self.assertEqual(FAKE['maintenance'], sot.is_maintenance)
         self.assertEqual(FAKE['last_error'], sot.last_error)
+        self.assertEqual(FAKE['lessee'], sot.lessee)
         self.assertEqual(FAKE['links'], sot.links)
         self.assertEqual(FAKE['maintenance_reason'], sot.maintenance_reason)
         self.assertEqual(FAKE['name'], sot.name)
@@ -197,7 +207,7 @@ class TestNode(base.TestCase):
 @mock.patch.object(node.Node, 'fetch', autospec=True)
 class TestNodeWaitForProvisionState(base.TestCase):
     def setUp(self):
-        super(TestNodeWaitForProvisionState, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock()
 
@@ -288,7 +298,7 @@ class TestNodeWaitForProvisionState(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeSetProvisionState(base.TestCase):
     def setUp(self):
-        super(TestNodeSetProvisionState, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion=None
@@ -329,6 +339,20 @@ class TestNodeSetProvisionState(base.TestCase):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
+    def test_deploy_with_configdrive_as_bytestring(self):
+        config_drive = base64.b64encode(b'foo')
+        result = self.node.set_provision_state(
+            self.session, 'active', config_drive=config_drive
+        )
+        self.assertIs(result, self.node)
+        self.session.put.assert_called_once_with(
+            'nodes/%s/states/provision' % self.node.id,
+            json={'target': 'active', 'configdrive': config_drive.decode()},
+            headers=mock.ANY,
+            microversion=None,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
+        )
+
     def test_rebuild_with_configdrive(self):
         result = self.node.set_provision_state(
             self.session, 'rebuild', config_drive='abcd'
@@ -420,7 +444,7 @@ class TestNodeSetProvisionState(base.TestCase):
 @mock.patch.object(node.Node, 'set_provision_state', autospec=True)
 class TestNodeCreate(base.TestCase):
     def setUp(self):
-        super(TestNodeCreate, self).setUp()
+        super().setUp()
         self.new_state = None
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.1'
@@ -532,7 +556,7 @@ class TestNodeCreate(base.TestCase):
 @mock.patch.object(node.Node, '_get_session', lambda self, x: x)
 class TestNodeVif(base.TestCase):
     def setUp(self):
-        super(TestNodeVif, self).setUp()
+        super().setUp()
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.28'
         self.session.log = mock.Mock()
@@ -562,13 +586,13 @@ class TestNodeVif(base.TestCase):
             json={'id': self.vif_id},
             headers=mock.ANY,
             microversion='1.28',
-            retriable_status_codes={503},
+            retriable_status_codes=[503],
         )
 
     def test_detach_vif_existing(self):
         self.assertTrue(self.node.detach_vif(self.session, self.vif_id))
         self.session.delete.assert_called_once_with(
-            'nodes/%s/vifs/%s' % (self.node.id, self.vif_id),
+            f'nodes/{self.node.id}/vifs/{self.vif_id}',
             headers=mock.ANY,
             microversion='1.28',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -578,7 +602,7 @@ class TestNodeVif(base.TestCase):
         self.session.delete.return_value.status_code = 400
         self.assertFalse(self.node.detach_vif(self.session, self.vif_id))
         self.session.delete.assert_called_once_with(
-            'nodes/%s/vifs/%s' % (self.node.id, self.vif_id),
+            f'nodes/{self.node.id}/vifs/{self.vif_id}',
             headers=mock.ANY,
             microversion='1.28',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -622,7 +646,7 @@ class TestNodeVif(base.TestCase):
 @mock.patch.object(node.Node, '_get_session', lambda self, x: x)
 class TestNodeValidate(base.TestCase):
     def setUp(self):
-        super(TestNodeValidate, self).setUp()
+        super().setUp()
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.28'
         self.node = node.Node(**FAKE)
@@ -681,7 +705,7 @@ class TestNodeValidate(base.TestCase):
 @mock.patch.object(node.Node, 'fetch', autospec=True)
 class TestNodeWaitForReservation(base.TestCase):
     def setUp(self):
-        super(TestNodeWaitForReservation, self).setUp()
+        super().setUp()
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.6'
         self.session.log = mock.Mock()
@@ -751,7 +775,7 @@ class TestNodeInjectNMI(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeSetPowerState(base.TestCase):
     def setUp(self):
-        super(TestNodeSetPowerState, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion=None
@@ -783,7 +807,7 @@ class TestNodeSetPowerState(base.TestCase):
 @mock.patch.object(node.Node, '_get_session', lambda self, x: x)
 class TestNodeMaintenance(base.TestCase):
     def setUp(self):
-        super(TestNodeMaintenance, self).setUp()
+        super().setUp()
         self.node = node.Node.existing(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter,
@@ -953,7 +977,7 @@ class TestNodeBootDevice(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeSetBootMode(base.TestCase):
     def setUp(self):
-        super(TestNodeSetBootMode, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion='1.1'
@@ -980,7 +1004,7 @@ class TestNodeSetBootMode(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeSetSecureBoot(base.TestCase):
     def setUp(self):
-        super(TestNodeSetSecureBoot, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion='1.1'
@@ -1007,7 +1031,7 @@ class TestNodeSetSecureBoot(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeTraits(base.TestCase):
     def setUp(self):
-        super(TestNodeTraits, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion='1.37'
@@ -1017,7 +1041,7 @@ class TestNodeTraits(base.TestCase):
     def test_node_add_trait(self):
         self.node.add_trait(self.session, 'CUSTOM_FAKE')
         self.session.put.assert_called_once_with(
-            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_FAKE'),
+            'nodes/{}/traits/{}'.format(self.node.id, 'CUSTOM_FAKE'),
             json=None,
             headers=mock.ANY,
             microversion='1.37',
@@ -1027,7 +1051,7 @@ class TestNodeTraits(base.TestCase):
     def test_remove_trait(self):
         self.assertTrue(self.node.remove_trait(self.session, 'CUSTOM_FAKE'))
         self.session.delete.assert_called_once_with(
-            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_FAKE'),
+            'nodes/{}/traits/{}'.format(self.node.id, 'CUSTOM_FAKE'),
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1039,7 +1063,7 @@ class TestNodeTraits(base.TestCase):
             self.node.remove_trait(self.session, 'CUSTOM_MISSING')
         )
         self.session.delete.assert_called_once_with(
-            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_MISSING'),
+            'nodes/{}/traits/{}'.format(self.node.id, 'CUSTOM_MISSING'),
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1061,7 +1085,7 @@ class TestNodeTraits(base.TestCase):
 @mock.patch.object(resource.Resource, 'patch', autospec=True)
 class TestNodePatch(base.TestCase):
     def setUp(self):
-        super(TestNodePatch, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion=None
@@ -1104,7 +1128,7 @@ class TestNodePatch(base.TestCase):
 @mock.patch.object(node.Node, 'fetch', autospec=True)
 class TestNodeWaitForPowerState(base.TestCase):
     def setUp(self):
-        super(TestNodeWaitForPowerState, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock()
 
@@ -1134,9 +1158,9 @@ class TestNodeWaitForPowerState(base.TestCase):
 @mock.patch.object(utils, 'pick_microversion', lambda session, v: v)
 @mock.patch.object(node.Node, 'fetch', lambda self, session: self)
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
-class TestNodePassthru(object):
+class TestNodePassthru:
     def setUp(self):
-        super(TestNodePassthru, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = node.Mock(
             spec=adapter.Adapter, default_microversion='1.37'
@@ -1269,3 +1293,48 @@ class TestNodeInventory(base.TestCase):
             microversion='1.81',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
+
+
+@mock.patch.object(node.Node, 'fetch', lambda self, session: self)
+@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
+class TestNodeFirmware(base.TestCase):
+    def setUp(self):
+        super().setUp()
+        self.node = node.Node(**FAKE)
+        self.session = mock.Mock(
+            spec=adapter.Adapter,
+            default_microversion='1.86',
+        )
+
+    def test_list_firmware(self):
+        node_firmware = {
+            "firmware": [
+                {
+                    "created_at": "2016-08-18T22:28:49.653974+00:00",
+                    "updated_at": "2016-08-18T22:28:49.653974+00:00",
+                    "component": "BMC",
+                    "initial_version": "v1.0.0",
+                    "current_version": "v1.2.0",
+                    "last_version_flashed": "v1.2.0",
+                },
+                {
+                    "created_at": "2016-08-18T22:28:49.653974+00:00",
+                    "updated_at": "2016-08-18T22:28:49.653974+00:00",
+                    "component": "BIOS",
+                    "initial_version": "v1.0.0",
+                    "current_version": "v1.1.5",
+                    "last_version_flashed": "v1.1.5",
+                },
+            ]
+        }
+        self.session.get.return_value.json.return_value = node_firmware
+
+        res = self.node.list_firmware(self.session)
+        self.assertEqual(node_firmware, res)
+
+        self.session.get.assert_called_once_with(
+            'nodes/%s/firmware' % self.node.id,
+            headers=mock.ANY,
+            microversion='1.86',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
+        )
