@@ -17,8 +17,6 @@ from unittest import mock
 
 import testtools
 
-import openstack.cloud
-from openstack.cloud import exc
 import openstack.cloud.openstackcloud as oc_oc
 from openstack import exceptions
 from openstack.object_store.v1 import _proxy
@@ -30,7 +28,7 @@ from openstack import utils
 
 class BaseTestObject(base.TestCase):
     def setUp(self):
-        super(BaseTestObject, self).setUp()
+        super().setUp()
 
         self.container = self.getUniqueString()
         self.object = self.getUniqueString()
@@ -198,7 +196,7 @@ class TestObject(BaseTestObject):
             ]
         )
         self.assertRaises(
-            openstack.cloud.OpenStackCloudException,
+            exceptions.SDKException,
             self.cloud.delete_container,
             self.container,
         )
@@ -231,7 +229,7 @@ class TestObject(BaseTestObject):
             [dict(method='POST', uri=self.container_endpoint, status_code=409)]
         )
         self.assertRaises(
-            openstack.cloud.OpenStackCloudException,
+            exceptions.SDKException,
             self.cloud.update_container,
             self.container,
             dict(foo='bar'),
@@ -284,7 +282,7 @@ class TestObject(BaseTestObject):
 
     def test_set_container_access_invalid(self):
         self.assertRaises(
-            openstack.cloud.OpenStackCloudException,
+            exceptions.SDKException,
             self.cloud.set_container_access,
             self.container,
             'invalid',
@@ -319,7 +317,7 @@ class TestObject(BaseTestObject):
         )
 
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             "Could not determine container access for ACL: invalid",
         ):
             self.cloud.get_container_access(self.container)
@@ -329,14 +327,14 @@ class TestObject(BaseTestObject):
             [dict(method='HEAD', uri=self.container_endpoint, status_code=404)]
         )
         with testtools.ExpectedException(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             "Container not found: %s" % self.container,
         ):
             self.cloud.get_container_access(self.container)
 
     def test_list_containers(self):
-        endpoint = '{endpoint}/'.format(endpoint=self.endpoint)
-        containers = [{u'count': 0, u'bytes': 0, u'name': self.container}]
+        endpoint = f'{self.endpoint}/'
+        containers = [{'count': 0, 'bytes': 0, 'name': self.container}]
 
         self.register_uris(
             [
@@ -356,7 +354,7 @@ class TestObject(BaseTestObject):
             self._compare_containers(a, b)
 
     def test_list_containers_exception(self):
-        endpoint = '{endpoint}/'.format(endpoint=self.endpoint)
+        endpoint = f'{self.endpoint}/'
         self.register_uris(
             [
                 dict(
@@ -368,9 +366,7 @@ class TestObject(BaseTestObject):
             ]
         )
 
-        self.assertRaises(
-            exc.OpenStackCloudException, self.cloud.list_containers
-        )
+        self.assertRaises(exceptions.SDKException, self.cloud.list_containers)
         self.assert_calls()
 
     @mock.patch.object(_proxy, '_get_expiration', return_value=13345)
@@ -602,11 +598,11 @@ class TestObject(BaseTestObject):
 
         objects = [
             {
-                u'bytes': 20304400896,
-                u'last_modified': u'2016-12-15T13:34:13.650090',
-                u'hash': u'daaf9ed2106d09bba96cf193d866445e',
-                u'name': self.object,
-                u'content_type': u'application/octet-stream',
+                'bytes': 20304400896,
+                'last_modified': '2016-12-15T13:34:13.650090',
+                'hash': 'daaf9ed2106d09bba96cf193d866445e',
+                'name': self.object,
+                'content_type': 'application/octet-stream',
             }
         ]
 
@@ -627,11 +623,11 @@ class TestObject(BaseTestObject):
 
         objects = [
             {
-                u'bytes': 20304400896,
-                u'last_modified': u'2016-12-15T13:34:13.650090',
-                u'hash': u'daaf9ed2106d09bba96cf193d866445e',
-                u'name': self.object,
-                u'content_type': u'application/octet-stream',
+                'bytes': 20304400896,
+                'last_modified': '2016-12-15T13:34:13.650090',
+                'hash': 'daaf9ed2106d09bba96cf193d866445e',
+                'name': self.object,
+                'content_type': 'application/octet-stream',
             }
         ]
 
@@ -660,7 +656,7 @@ class TestObject(BaseTestObject):
             ]
         )
         self.assertRaises(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             self.cloud.list_objects,
             self.container,
         )
@@ -798,7 +794,7 @@ class TestObject(BaseTestObject):
         )
 
         self.assertRaises(
-            openstack.cloud.OpenStackCloudException,
+            exceptions.SDKException,
             self.cloud.get_object,
             self.container,
             self.object,
@@ -883,7 +879,7 @@ class TestObject(BaseTestObject):
 
 class TestObjectUploads(BaseTestObject):
     def setUp(self):
-        super(TestObjectUploads, self).setUp()
+        super().setUp()
 
         self.content = self.getUniqueString().encode('latin-1')
         self.object_file = tempfile.NamedTemporaryFile(delete=False)
@@ -1110,7 +1106,7 @@ class TestObjectUploads(BaseTestObject):
                         index=index,
                     ),
                     status_code=201,
-                    headers=dict(Etag='etag{index}'.format(index=index)),
+                    headers=dict(Etag=f'etag{index}'),
                 )
                 for index, offset in enumerate(
                     range(0, len(self.content), max_file_size)
@@ -1232,7 +1228,7 @@ class TestObjectUploads(BaseTestObject):
                         index=index,
                     ),
                     status_code=201,
-                    headers=dict(Etag='etag{index}'.format(index=index)),
+                    headers=dict(Etag=f'etag{index}'),
                 )
                 for index, offset in enumerate(
                     range(0, len(self.content), max_file_size)
@@ -1391,7 +1387,7 @@ class TestObjectUploads(BaseTestObject):
                         index=index,
                     ),
                     status_code=201,
-                    headers=dict(Etag='etag{index}'.format(index=index)),
+                    headers=dict(Etag=f'etag{index}'),
                 )
                 for index, offset in enumerate(
                     range(0, len(self.content), max_file_size)
@@ -1511,7 +1507,7 @@ class TestObjectUploads(BaseTestObject):
         self.cloud.image_api_use_tasks = True
 
         self.assertRaises(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             self.cloud.create_object,
             container=self.container,
             name=self.object,
@@ -1594,7 +1590,7 @@ class TestObjectUploads(BaseTestObject):
         )
 
         self.assertRaises(
-            exc.OpenStackCloudException,
+            exceptions.SDKException,
             self.cloud.create_object,
             container=self.container,
             name=self.object,
